/*

 AWD file loading example in Away3d

 Demonstrates:

 How to use the Loader3D object to load an embedded internal 3ds model.
 How to map an external asset reference inside a file to an internal embedded asset.
 How to extract material data and use it to set custom material properties on a model.

 Code by Rob Bateman
 rob@infiniteturtles.co.uk
 http://www.infiniteturtles.co.uk

 This code is distributed under the MIT License

 Copyright (c) The Away Foundation http://www.theawayfoundation.org

 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the “Software”), to deal
 in the Software without restriction, including without limitation the rights
 to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 copies of the Software, and to permit persons to whom the Software is
 furnished to do so, subject to the following conditions:

 The above copyright notice and this permission notice shall be included in
 all copies or substantial portions of the Software.

 THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.

 */
package;

import away3d.cameras.Camera3D;
import away3d.containers.ObjectContainer3D;
import away3d.events.LoaderEvent;
import away3d.lights.LightBase;
import away3d.primitives.SkyBox;
import openfl.display.Sprite;
import openfl.net.URLRequest;
import openfl.events.Event;
import openfl.geom.Vector3D;
import openfl.Vector;

import away3d.containers.View3D;
import away3d.lights.DirectionalLight;
import away3d.materials.lightpickers.StaticLightPicker;
import away3d.entities.Mesh;
import away3d.primitives.SphereGeometry;
import away3d.library.Asset3DLibrary;
import away3d.loaders.parsers.AWDParser;
import away3d.events.Asset3DEvent;
import away3d.library.assets.IAsset;
import away3d.library.assets.Asset3DType;
import away3d.materials.TextureMaterial;
import away3d.utils.Cast;
import openfl.display.BitmapData;

import openfl.Assets;

class Main extends Sprite
{
	//engine variables
	var _view:View3D;

	//light objects
	var _light:DirectionalLight;
	var _lightPicker:StaticLightPicker;
	var _direction:Vector3D;

	//scene objects
	var loaded:Bool = false;

	/**
	 * Constructor
	 */
	public function new()
	{
		super();
		init();
	}

	/**
	 * Global initialise function
	 */
	private function init():Void
	{
		initEngine();
		//initLights();
		initMaterials();
		initObjects();
		initListeners();
   }

	/**
	 * Initialise the engine
	 */
	private function initEngine():Void
	{
		_view = new View3D();
		this.addChild(_view);

		//set the background of the view to something suitable
		_view.backgroundColor = 0x1e2125;
 
		// stats
		this.addChild(new away3d.debug.AwayFPS(_view, 10, 10, 0xffffff, 3));
	}

	/**
	 * Initialise the lights
	 */
	private function initLights():Void
	{
		//create the light for the scene
		_light = new DirectionalLight();
		_light.color = 0x683019;
		_light.direction = new Vector3D(1, 0, 0);
		_light.ambient = 0.5;
		_light.ambientColor = 0x30353b;
		_light.diffuse = 2.8;
		_light.specular = 1.8;
		_light.y = 33;
		_view.scene.addChild(this._light);

		//create the lightppicker for the material
		_lightPicker = new StaticLightPicker([this._light]);
	}

	/**
	 * Initialise the materials
	 */
	private function initMaterials():Void
	{
	}

	/**
	 * Initialise the scene objects
	 */
	private function initObjects():Void
	{
		_view.scene.addChild(new Mesh(new SphereGeometry(0)));
	}

	/**
	 * Initialise the listeners
	 */
	private function initListeners():Void
	{
		stage.addEventListener(Event.RESIZE, onResize);

		onResize();

		addEventListener(Event.ENTER_FRAME, onEnterFrame);

		Asset3DLibrary.enableParser(AWDParser);

		Asset3DLibrary.addEventListener(Asset3DEvent.ASSET_COMPLETE, onAssetComplete);
		Asset3DLibrary.addEventListener(LoaderEvent.RESOURCE_COMPLETE, onLoadAssetComplete);

		#if SingleMesh
		trace ("SingleMesh");
		Asset3DLibrary.loadData(Assets.getBytes('Model/ModelSingleMesh.awd'));
		#elseif MultipleMeshes
		trace ("MultipleMeshes");
		Asset3DLibrary.loadData(Assets.getBytes('Model/ModelMultipleMeshes.awd'));
		#else
		Asset3DLibrary.loadData(Assets.getBytes('Model/Model.awd'));
		#end
	}

	/**
	 * Navigation and render loop
	 */
	
	private function onLoadAssetComplete(e:Dynamic):Void {
		loaded = true;
	}
	 
	private function onEnterFrame(e:Event):Void
	{
		if (loaded) {
			_view.render();
			//_view.camera.rotationY += 1;
		}
	}

	/**
	 * Listener function for asset complete event on loader
	 */
	private function onAssetComplete (event:Asset3DEvent)
	{
		var asset:IAsset = event.asset;

		switch (asset.assetType)
		{
			case Asset3DType.CONTAINER :
				if (cast(asset, ObjectContainer3D).parent == null) _view.scene.addChild(cast(asset, ObjectContainer3D));
			case Asset3DType.MESH :
				var mesh:Mesh = cast(asset, Mesh);
				if (mesh.parent == null) _view.scene.addChild(mesh);
			case Asset3DType.CAMERA :
				_view.camera = cast asset;
			case Asset3DType.LIGHT:
				var l:LightBase = cast asset;
				if (l.parent == null) _view.scene.addChild(l);
		}
	}

	/**
	 * stage listener for resize events
	 */
	private function onResize(event:Event = null):Void
	{
		_view.width = stage.stageWidth;
		_view.height = stage.stageHeight;
	}
}

